@@ -1,4 +1,6 @@
+from django import forms
 from django.contrib import admin
+from django.core.exceptions import ValidationError
 from djangoldp.admin import DjangoLDPAdmin
 from .models import (
     Application,
@@ -142,6 +144,13 @@ class ComponentExtensionAdmin(DjangoLDPAdmin):
     ordering = ["base_component__application__slug", "component__slug"]
 
 
+class DeploymentAdmin(DjangoLDPAdmin):
+    list_display = ("application", "date", "status")
+    exclude = ("urlid", "slug", "is_backlink", "allow_create_backlink")
+    search_fields = ["urlid", "application__slug", "application__friendly_name"]
+    ordering = ["application__slug", "date"]
+
+
 admin.site.register(Application, ApplicationAdmin)
 admin.site.register(Federation, EmptyAdmin)
 admin.site.register(ApplicationGraphics, EmptyAdmin)
@@ -153,4 +162,4 @@ admin.site.register(ApplicationComponentParameter, EmptyAdmin)
 admin.site.register(ApplicationPackageParameter, EmptyAdmin)
 admin.site.register(ComponentExtension, ComponentExtensionAdmin)
 admin.site.register(ComponentExtensionParameter, EmptyAdmin)
-admin.site.register(Deployment, EmptyAdmin)
+admin.site.register(Deployment, DeploymentAdmin)
