@@ -1,8 +1,26 @@
+from django.http import JsonResponse
 from rest_framework import serializers, viewsets
 from rest_framework_yaml.parsers import YAMLParser
 from rest_framework_yaml.renderers import YAMLRenderer
 from .models import Application, Federation
 from djangoldp_component.models import Component, Package
+from django.conf import settings
+
+
+ANSIBLE_SERVERS = set({"127.0.0.1"})
+
+
+if hasattr(settings, "ANSIBLE_SERVERS"):
+    ANSIBLE_SERVERS = ANSIBLE_SERVERS.union(getattr(settings, "ANSIBLE_SERVERS"))
+
+
+def get_client_ip(request):
+    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
+    if x_forwarded_for:
+        ip = x_forwarded_for.split(",")[0]
+    else:
+        ip = request.META.get("REMOTE_ADDR")
+    return ip
 
 
 def format(value):
@@ -112,7 +130,7 @@ class ApplicationDetailSerializer(serializers.HyperlinkedModelSerializer):
             insertComponent = {
                 "type": component.name,
                 "route": format(component.preferred_route),
-                "parameters": [],
+                "parameters": {},
                 "extensions": [],
             }
             if component.auto_import:
@@ -122,24 +140,20 @@ class ApplicationDetailSerializer(serializers.HyperlinkedModelSerializer):
                 if parameter.key == "route":
                     insertComponent["route"] = format(parameter.value)
                 else:
-                    i = {}
-                    i[parameter.key] = format(parameter.value)
-                    insertComponent["parameters"].append(i)
+                    insertComponent["parameters"][parameter.key] = format(parameter.value)
                 keys.append(parameter.key)
             for parameter in component.parameters.all():
                 if not parameter.key in keys:
                     if parameter.key == "route":
                         insertComponent["route"] = format(parameter.default)
                     else:
-                        i = {}
-                        i[parameter.key] = format(parameter.default)
-                        insertComponent["parameters"].append(i)
+                        insertComponent["parameters"][parameter.key] = format(parameter.default)
             for extensionComponent in applicationComponent.obj.extensions.all():
                 extension = Component.objects.get(id=extensionComponent.component_id)
                 componentExtension = {
                     "type": extension.name,
                     "route": format(component.preferred_route),
-                    "parameters": [],
+                    "parameters": {},
                 }
                 if extension.auto_import:
                     componentExtension["experimental"] = ["routing"]
@@ -148,18 +162,14 @@ class ApplicationDetailSerializer(serializers.HyperlinkedModelSerializer):
                     if parameter.key == "route":
                         componentExtension["route"] = format(parameter.value)
                     else:
-                        i = {}
-                        i[parameter.key] = format(parameter.value)
-                        componentExtension["parameters"].append(i)
+                        componentExtension["parameters"][parameter.key] = format(parameter.value)
                     keys.append(parameter.key)
                 for parameter in extension.parameters.all():
                     if not parameter.key in keys:
                         if parameter.key == "route":
                             componentExtension["route"] = format(parameter.default)
                         else:
-                            i = {}
-                            i[parameter.key] = format(parameter.default)
-                            componentExtension["parameters"].append(i)
+                            componentExtension["parameters"][parameter.key] = format(parameter.default)
                 insertComponent["extensions"].append(componentExtension)
             serialized["apps"]["hosts"][application["slug"]]["components"].append(
                 insertComponent
@@ -171,20 +181,16 @@ class ApplicationDetailSerializer(serializers.HyperlinkedModelSerializer):
             insertDependency = {
                 "distribution": package.distribution,
                 "module": package.module,
-                "parameters": [],
+                "parameters": {},
             }
             keys = []
             for parameter in applicationPackage.obj.parameters.all():
                 if not parameter.key in keys:
-                    i = {}
-                    i[parameter.key] = format(parameter.value)
-                    insertDependency["parameters"].append(i)
+                    insertDependency["parameters"][parameter.key] = format(parameter.value)
                     keys.append(parameter.key)
             for parameter in package.parameters.all():
                 if not parameter.key in keys:
-                    i = {}
-                    i[parameter.key] = format(parameter.default)
-                    insertDependency["parameters"].append(i)
+                    insertDependency["parameters"][parameter.key] = format(parameter.default)
             insertDependencies.append(insertDependency)
         serialized["apps"]["hosts"][application["slug"]][
             "packages"
@@ -226,3 +232,25 @@ class ApplicationDetailViewSet(viewsets.ModelViewSet):
     lookup_field = "slug"
     parser_classes = (YAMLParser,)
     renderer_classes = (YAMLRenderer,)
+
+
+def mark_as_done(request, slug):
+    if request.method == "GET" and get_client_ip(request) in ANSIBLE_SERVERS:
+        application = Application.objects.get(slug=slug)
+        for deploy in application.deployments.filter(status="Todo"):
+            deploy.status = "Done"
+            deploy.save()
+        return JsonResponse({"status": "success"}, status=200)
+    else:
+        return JsonResponse({"status": "invalid request"}, status=400)
+
+
+def mark_as_failed(request, slug):
+    if request.method == "GET" and get_client_ip(request) in ANSIBLE_SERVERS:
+        application = Application.objects.get(slug=slug)
+        for deploy in application.deployments.filter(status="Todo"):
+            deploy.status = "Failed"
+            deploy.save()
+        return JsonResponse({"status": "success"}, status=200)
+    else:
+        return JsonResponse({"status": "invalid request"}, status=400)
